!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function A43phiAQggmppm(j1,j2,j3,j4,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: A43phiAQggmppm
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/zprods_decl.f'
      complex(dp):: A43phiAQggmpmp_unsym,A43phiAQggmppm_unsym
      integer:: j1,j2,j3,j4
      A43phiAQggmppm=
     & +A43phiAQggmppm_unsym(j1,j2,j3,j4,za,zb)
     & +A43phiAQggmpmp_unsym(j1,j2,j4,j3,za,zb)
      return
      end

c--- The expression below corresponds to one half of A4;3:
c---   Aleft(a1,q2,g3+,g4-)+Aright(a1,q2,g3+,g4-)+Aleft(a1,g3+,q2,g4-)

c--- It is obtained by summing the expressions given in:
c---  L.~J.~Dixon and Y.~Sofianatos,
c---  %``Analytic one-loop amplitudes for a Higgs boson plus four partons,''
c---  arXiv:0906.0008 [hep-ph].
      function A43phiAQggmppm_unsym(j1,j2,j3,j4,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: A43phiAQggmppm_unsym
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/scale.f'
      include 'src/Inc/epinv.f'
      complex(dp):: V5L,V6l,A0phiAQggmppm,lnrat
      complex(dp):: l13,l34,l12,l24,Lsm1DS,Lsm1_2me
      real(dp):: s3,mhsq,s123,s234,s341,s412
      integer:: j1,j2,j3,j4
      s3(j1,j2,j3)=s(j1,j2)+s(j1,j3)+s(j2,j3)

      s123=s3(j1,j2,j3)
      s234=s3(j2,j3,j4)
      s341=s3(j3,j4,j1)
      s412=s3(j4,j1,j2)
      mhsq=s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)      
      l12=lnrat(musq,-s(j1,j2))
      l13=lnrat(musq,-s(j1,j3))
      l24=lnrat(musq,-s(j2,j4))
      l34=lnrat(musq,-s(j3,j4))

c--- This is the same function as in the mpmm amplitude
      V5L=
     & -epinv**2-epinv*l12-0.5_dp*l12**2
     & -epinv**2-epinv*l34-0.5_dp*l34**2
     & +epinv**2+epinv*l13+0.5_dp*l13**2
     & +epinv**2+epinv*l24+0.5_dp*l24**2

c--- These are additional boxes
      V6L=
     & +Lsm1_2me(s341,s123,s(j1,j3),mhsq) 
     & +Lsm1_2me(s412,s234,s(j2,j4),mhsq) 
     & -Lsm1_2me(s412,s123,s(j1,j2),mhsq) 
     & -Lsm1_2me(s234,s341,s(j3,j4),mhsq) 
     & +Lsm1DS(s(j2,j4),s(j4,j1),s412)
     & +Lsm1DS(s(j1,j3),s(j3,j2),s123)
     & +Lsm1DS(s(j2,j4),s(j2,j3),s234)
     & +Lsm1DS(s(j1,j3),s(j1,j4),s341)
      
      A43phiAQggmppm_unsym=
     &  A0phiAQggmppm(j1,j2,j3,j4,za,zb)
     &  *(V5L+V6L
     &    -Lsm1DS(s(j2,j3),s(j3,j4),s234)
     &    -Lsm1DS(s(j4,j1),s(j1,j2),s3(j4,j1,j2)))

     & +(za(j1,j4)**3/(za(j1,j2)*za(j3,j4)*za(j1,j3))
     &  +za(j1,j4)**2/(za(j2,j3)*za(j1,j3)))
     &  *(Lsm1DS(s(j1,j2),s(j2,j3),s123)
     &   +Lsm1DS(s(j3,j4),s(j4,j1),s3(j3,j4,j1)))
     
      return
      end
