!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine coupling
      implicit none
      include 'src/Inc/types.f'
c--- initialize electroweak couplings and calculate alpha-s; this
c--- must be called at the beginning of "chooser" to enable it to
c--- set up all the variables (e.g. twidth). Once nflav is set,
c--- alpha-s will be determined again (in coupling2).
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/masses.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/couple.f'
      include 'src/Inc/scale.f'
      include 'src/Inc/nlooprun.f'
      include 'src/Inc/ewinput.f'
      include 'src/Inc/nflav.f'
      include 'src/Inc/b0.f'
      include 'src/Inc/kpart.f'
      include 'src/Inc/verbose.f'
      include 'src/Inc/mpicommon.f'
      include 'src/Inc/kprocess.f'
      include 'src/Inc/nproc.f'
      integer:: i
      real(dp):: aemmz,alphas,cmass,bmass,lotopdecaywidth
      real(dp) :: topwidth,nloratiotopdecay
      character*3 inlabel(10)
      character*5 tworder
      common/qmass/cmass,bmass
      common/em/aemmz
      logical, save :: first = .true.

c--- blank out labels that indicate input parameters
      do i=1,10
        inlabel(i)='   '
      enddo
      inlabel(3)='(+)'
      inlabel(4)='(+)'
      inlabel(8)='(+)'

      if (ewscheme == -1) then
c--- This is the MCFM default, corresponding to an effective
c--- field theory approach valid for scales below the top-mass
C--- (see Georgi, Nucl. Phys. B 363 (1991) 301).
c--- There are 4 inputs here instead of the usual 3 ...
         Gf = Gf_inp
         aemmz  = aemmz_inp
         wmass  = wmass_inp
         zmass  = zmass_inp
         inlabel(5)='(+)'
         inlabel(6)='(+)'
         inlabel(1)='(+)'
         inlabel(2)='(+)'
         inlabel(8)='   '
c--- ... and as result, both xw and mtop are derived
c--- (using wmass=zmass*sqrt(rho)*cos(theta_w)
c---    and rho=1+3._dp*aemmz/16._dp/pi/xw*(mt/wmass)**2 )
         xw  = fourpi*aemmz/(8._dp*wmass**2*Gf/rt2)
         mt  = sqrt(16._dp*pisq/3._dp/rt2/Gf*(
     &          wmass**2/zmass**2/(1._dp-xw)-1._dp))

      elseif (ewscheme == 0) then
c------------------------------------------------------------
c     option=0 : MadEvent default (= AlpGen with iewopt=2)
c------------------------------------------------------------

c-- equal to the input values
         xw  = xw_inp
         aemmz  = aemmz_inp
         zmass  = zmass_inp
         inlabel(7)='(+)'
         inlabel(6)='(+)'
         inlabel(1)='(+)'
c-- derived
         wmass  = zmass * sqrt( One - xw )
         Gf = aemmz * Fourpi/xw/(8._dp*wmass**2/Rt2)

      elseif (ewscheme == 1) then
c-----------------------------------------------------
c     option=1 : LUSIFER and AlpGen (iewopt=3) default
c-----------------------------------------------------

c-- equal to the input values
         zmass  = zmass_inp
         wmass  = wmass_inp
         Gf = Gf_inp
         inlabel(1)='(+)'
         inlabel(2)='(+)'
         inlabel(5)='(+)'
c-- derived
         xw  = One-(wmass/zmass)**2
         aemmz  = Rt2*Gf*wmass**2*xw/pi

      elseif (ewscheme == 2) then
c-------------------------------------------------------------------
c     option=2 : W and Z mass are derived from couplings
c-------------------------------------------------------------------

c-- equal to the input values
         Gf = Gf_inp
         aemmz  = aemmz_inp
         xw  = xw_inp
         inlabel(5)='(+)'
         inlabel(6)='(+)'
         inlabel(7)='(+)'
c-- derived
         wmass  = sqrt(aemmz*pi/xw/Gf/Rt2)
         zmass  = wmass/sqrt(One-xw)

      elseif (ewscheme == 3) then
c-----------------------------------------------------------------
c     option=3 : USER choice : you should know what you're doing!!
c-----------------------------------------------------------------
         Gf = Gf_inp
         aemmz  = aemmz_inp
         xw  = xw_inp
         wmass  = wmass_inp
         zmass  = zmass_inp
         inlabel(5)='(+)'
         inlabel(6)='(+)'
         inlabel(7)='(+)'
         inlabel(1)='(+)'
         inlabel(2)='(+)'

      else
         write(6,*) 'ewscheme=',ewscheme,' is not a valid input.'
         stop
      endif

c--- Now set up the other derived parameters
      gwsq=fourpi*aemmz/xw
      esq=gwsq*xw
      gw=sqrt(gwsq)
      call couplz(xw)

c--- Calculate the appropriate Higgs vacuum expectation value.
c--- This vevsq is defined so that gwsq/(4*wmass**2)=Gf*rt2=1/vevsq
c--- (ie differs from definition in ESW)
      vevsq=1._dp/rt2/Gf

c--- set up the beta-function
      b0=(xn*11._dp-2._dp*nflav)/6._dp

c--- initialize the pdf set
      nlooprun=0
      call pdfwrap      

      cmass=sqrt(mcsq)
      bmass=sqrt(mbsq)
      musq=scale**2
 
c--- set the number of loops to use in the running of alpha_s
c--- if it hasn't been set by pdfwrap already
      if (nlooprun == 0) then
        if (kpart==klord) then
          nlooprun=1
        else
          nlooprun=2
        endif
      endif

c--- initialize alpha_s
      as=alphas(abs(scale),amz,nlooprun)

      ason2pi=as/twopi
      ason4pi=as/fourpi
      gsq=fourpi*as

c--- Set-up twidth, using LO formula everywhere
      if ((nproc == 161) .or. (nproc == 165)) then
          write (*,*) "Top decay width fixed at LO with mb=0, wwidth=0"
          write (*,*) "Assembly routine will take into account higher order pieces."
          twidth=lotopdecaywidth(mt,0d0,wmass,0d0)
          tworder='(LO) '
          ! from 1210.2808
          ! LO 1.4806
          ! NLO 1.5109
          ! NNLO 1.5415
      else
          twidth=lotopdecaywidth(mt,mb,wmass,wwidth)
          if (kpart /= klord) then
              twidth = twidth*(1 + nloratiotopdecay(mt,mb,wmass,wwidth))
              tworder = '(NLO)'
          endif
      endif

      if (verbose .and. rank == 0 .and. first ) then
      write(6,*) '************** Electroweak parameters **************'
      write(6,*) '*                                                  *'
      write(6,75) 'zmass',inlabel(1),zmass,'wmass',inlabel(2),wmass
      write(6,75) 'zwidth',inlabel(3),zwidth,'wwidth',inlabel(4),wwidth
      write(6,76) 'Gf',inlabel(5),gf,'1/aemmz',inlabel(6),1._dp/aemmz
      write(6,75) 'xw',inlabel(7),xw,'mtop',inlabel(8),mt
      write(6,75) 'gwsq',inlabel(9),gwsq,'esq',inlabel(10),esq
      write(6,77) 'top width',twidth,tworder
      write(6,78) 'mb',mb,'mc',mc
      write(6,*) '*                                                  *'
      write(6,*) '* Parameters marked (+) are input, others derived  *'
      write(6,*) '****************************************************'
      endif

   75 format(' * ',a6,a3,f13.7,3x,a7,a3,f12.7,'  *')
   76 format(' * ',a6,a3,d13.6,3x,a7,a3,f12.7,'  *')
   77 format(' * ',a9,f13.7,1x,a5,19x,'  *')
   78 format(' * ',a5,4x,f13.7,6x,a4,2x,f13.7,'  *')

      first = .false.
      
      return
      end


      block data wsalam1
      implicit none
      include 'src/Inc/types.f'
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/ewcharge.f'
      data Q(-5)/+0.333333333333333_dp/
      data Q(-4)/-0.666666666666667_dp/
      data Q(-3)/+0.333333333333333_dp/
      data Q(-2)/-0.666666666666667_dp/
      data Q(-1)/+0.333333333333333_dp/
      data Q(0)/+0._dp/
      data Q(+1)/-0.333333333333333_dp/
      data Q(+2)/+0.666666666666667_dp/
      data Q(+3)/-0.333333333333333_dp/
      data Q(+4)/+0.666666666666667_dp/
      data Q(+5)/-0.333333333333333_dp/
      data tau/1._dp,-1._dp,1._dp,-1._dp,1._dp,0._dp,-1._dp,1._dp,-1._dp,1._dp,-1._dp/
      end 



