!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine mcfm_init()
          use omp_lib
          use mod_qcdloop_c
          use parseinput
          use MCFMStorage
          use MCFMBenchmark, only : setupBenchmark
          use MCFMSetupPlots, only: setup_plots
          use MCFMPlotting, only : plots_allocate
          use iso_fortran_env
      implicit none
************************************************************************
*                                                                      *
*  This routine should initialize any necessary variables and          *
*  perform the usual print-outs                                        *
*                                                                      *
************************************************************************
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/cutoff.f'
      include 'src/Inc/limits.f'
      include 'src/Inc/npart.f'
      include 'src/Inc/phasemin.f'
      include 'src/Inc/facscale.f'
      include 'src/Inc/scale.f'
      include 'src/Inc/verbose.f'
      include 'src/Inc/phot_dip.f'
      include 'src/Inc/includect.f'
      include 'TensorReduction/Include/TRtensorcontrol.f'
      include 'src/Inc/frag.f'
      include 'src/Inc/energy.f'
      include 'TensorReduction/Include/TRmaxindex.f'
      include 'src/Inc/masses.f'
      include 'src/Inc/nflav.f'
      include 'src/Inc/epinv.f'
      include 'src/Inc/epinv2.f'
      include 'src/Inc/mpicommon.f'
      include 'src/Inc/nproc.f'
      include 'src/Inc/runname.f'

      include 'src/Inc/rtsmin.f'
      real(dp):: p1ext(4),p2ext(4),p(mxpart,4),val
      integer:: j,k
      common/pext/p1ext,p2ext
      data p/mxpart*3._dp,mxpart*4._dp,mxpart*0._dp,mxpart*5._dp/

      integer :: nplotmax
      common/nplotmax/nplotmax

      character(len=:), allocatable :: cmd
      character(len=cfg_string_len) :: benchmark

      logical :: newPlotting

!$omp threadprivate(/pext/)

c initialize qcdloop cache for all omp threads
!$omp parallel
      call qlcachesize(100)
!$omp end parallel

*Initialise data in commonblock /pvmaxindex/
      maxcindex=3
      maxdindex=4
      maxeindex=5

      if (rank.eq.0) call banner

      if (rank == 0 .and. world_size > 1) then
          write (*,*) ""
#ifdef HAVE_MPI
          write (*,*) "Running MCFM with ", world_size, " MPI processes"
#elif HAVE_COARRAY
          write (*,*) "Running MCFM with ", world_size, " Coarray images"
#endif
          write (*,*) ""
      endif
      if (rank == 0) then
          write (*,*) ""
          write (*,*) "Running MCFM with ", omp_get_max_threads(), " OMP threads" 
          write (*,*) ""
      endif

      if (rank == 0) then
          write (*,*) ""
          write (*,*) "MCFM compiled with ", compiler_version(), " using the options ",
     &          compiler_options()
          write (*,*) ""
      endif

      if (rank ==0) then
          allocate(character(len=1024) :: cmd)
          call get_command(cmd)
          write (*,*) ""
          write (*,*) "Running MCFM as ", trim(cmd)
          write (*,*) ""
          deallocate(cmd)
      endif

      if (rank.ge.1) verbose=.false.

      ! TODO: add this to configuration file
* Initialize parameter settings
      call mdata

      epinv=1.e1_dp
      epinv2=1.e1_dp


      call default_config()

      call cfg_update_from_arguments(cfg)
      call cfg_sort(cfg)

      if (cfg_var_configadded(cfg, "extra%benchmark")) then
          call cfg_get(cfg, "extra%benchmark", benchmark)
          write(6,*)
          write(6,*) '****************************************'
          write(6,*) '*      Running in benchmark mode       *'
          write(6,*) '* bench = ', benchmark
          write(6,*) '****************************************'
          write(6,*)
          call setupBenchmark()
      endif

      call read_config()
      !call cfg_write(cfg, "stdout")

      if (verbose) then
      write(6,*)
      write(6,*) '****************************************'
      write(6,*) '*     Cross section in femtobarns      *'
      write(6,*) '****************************************'
      write(6,*)
      endif

* Counter-terms for radiation in top decay should be included
      includect=.true.
      
* Set-up incoming beams and PS integration cut-offs
c--- Note: version 6.4 onwards, scale cutoff with c.o.m. energy
c--- Note: since Sep. 2018 no longer scale since it is dimensionless
!      cutoff=cutoff*(sqrts/2000._dp)**2
      rtsmin=min(rtsmin,sqrt(wsqmin+cutoff))
      rtsmin=min(rtsmin,sqrt(bbsqmin+cutoff))
      taumin=(rtsmin/sqrts)**2
      xmin=1.e-8_dp

      p1ext(4)=-half*sqrts
      p1ext(1)=0._dp
      p1ext(2)=0._dp
      p1ext(3)=-half*sqrts

      p2ext(4)=-half*sqrts
      p2ext(1)=0._dp
      p2ext(2)=0._dp
      p2ext(3)=+half*sqrts

* Set-up run name
      call setrunname(scale,facscale,cfg%config_directory)

* npart=9 is a dummy value, to ensure that all histograms are included
      npart=9
      val=1.e-15_dp   
      nprocbelow = nproc

      call cfg_get(cfg,"histogram%newstyle",newPlotting)

      if (newPlotting .or. nproc == 161 .or. nproc == 165) then
          call setup_plots()
          call plots_allocate()
      else

      ! This first call has two purposes:
      ! 1. determine nplotmax for histogram allocation.
      ! 2. must be run in parallel so that each thread initializes
      !    in "tagbook" mode and takes runs fully in "tagplot"
      !    for subsequent calls in the integration routines
!$omp parallel
      call nplotter(p,val,val**2,1)
!$omp end parallel

!$omp parallel
      call initHistogramStorage(nplotmax)
!$omp end parallel
      call initMasterStorage(nplotmax)

      endif

      p(:,:) = 0._dp
       
* Initialize flag for photon fragmentation dipoles
      phot_dip(:)=.false.
      fragint_mode=.false. 
* Initialize integer:: used in TensorReduction to zero
      TRtensorcontrol=0
      
      end
            
