!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>

module MCFMPDF
#ifdef HAVE_LHAPDF
    use LHAPDF
#endif
    use MCFMStorage, only: selectpdfs
    implicit none

    public :: fdist

    private

    contains

#ifndef HAVE_LHAPDF

    subroutine fdist(ih_call,xx,xxmu,ffx, ibeam_in)
    implicit none
    include 'src/Inc/types.f'
    include 'src/Inc/pdlabel.f'
    include 'src/Inc/kprocess.f'
    include 'src/Inc/mpicommon.f'
    include 'src/Inc/noglue.f'
    real(dp),intent(in)::xx,xxmu
    integer,intent(in)::ih_call
    real(dp),intent(out)::ffx(-5:5)
    integer, intent(in), optional :: ibeam_in
    real(dp) :: fx(-5:5),xZ,xA,eks98r,xmu_safe,x,xmu
    real(dp) :: u_val,d_val,u_sea,d_sea,s_sea,c_sea,b_sea,gluon
    real(dp) :: Ctq3df,Ctq4Fn,Ctq5Pdf,Ctq6Pdf,Ctq5L,CT10Pdf, &
    CT14Pdf,CT14Pdfqed
    real(dp) :: fxnnpdf(-6:7)
    integer :: mode,Iprtn,ih,iZ,iA,Irt
    integer :: i
    logical :: first,nucleon
!--- extra variables for MSTW08 implementation
    character(72) :: prefix,checkpath
    real(dp) :: str,sbar,chm,cbar,bot,bbar,photon
    data first/ .TRUE. /
    save first

    logical :: dummypdf
    common/dummypdf/dummypdf

    integer :: ibeam_use

    if (present(ibeam_in)) then
        ibeam_use = ibeam_in
    else
        ibeam_use = 1
    endif
!---  ih1=+1 proton
!---  ih1=-1 pbar

!---  Extended 5/24/05 to calculate nucleon parton distributions
!---  for a nucleus of atomic number Z and mass A via:
!---   K.J. Eskola, V.J. Kolhinen and C.A. Salgado,
!---   "The scale dependent nuclear effects in parton distributions for
!---   practical applications", Eur. Phys. J. C9 (1999) 61,
!---   JYFL-8/98, US-FT/14-98, hep-ph/9807297.

     if (dummypdf) then
         do i=-5,5
             if (selectpdfs(ibeam_use,i)) then
                 ffx(i) = xx**0.1d0*(1-xx)
             endif
         enddo
         return
     endif

    x=dble(xx)
    xmu=dble(xxmu)
    nucleon= .FALSE. 
    if     (ih_call > 1000d0) then
    !--- nucleon distribution functions
        ih=1
        nucleon= .TRUE. 
        iA=mod(ih_call,1000)
        iZ=(ih_call-iA)/1000
        xA=real(iA,dp)
        xZ=real(iZ,dp)
        if (first) then
        ! omp master
            if (rank == 0) then
                write(6,*)
                write(6,*)'******************* Nucleon beam *******************'
                write(6,*)'*                                                  *'
                write(6,76) iZ,iA
                write(6,*)'****************************************************'
            endif
        ! omp end master
            first= .FALSE. 
        endif
    elseif (abs(ih_call) /= 1) then
    !--- error
        write(6,*) 'Input beam ih=',ih,' does not make sense!'
        stop
    else
    !--- parton distribution functions
        ih=ih_call
    endif
              
!---set to zero if x out of range
    if ((x <= 0d0) .OR. (x >= 1d0)) then
        do Iprtn=-5,5
            fx(Iprtn)=0d0
        enddo
        return
    endif
     
    if      ((pdlabel(1:5) == 'mstw8') .OR. &
    (pdlabel(1:4) == 'MMHT') ) then
        if         (pdlabel == 'mstw8lo') then
            prefix = checkpath('Pdfdata/mstw2008lo') ! LO grid
        elseif     (pdlabel == 'mstw8nl') then
            prefix = checkpath('Pdfdata/mstw2008nlo') ! NLO grid
        elseif     (pdlabel == 'mstw8nn') then
            prefix = checkpath('Pdfdata/mstw2008nnlo') ! NNLO grid
        elseif     (pdlabel == 'MMHT_lo') then
            prefix = checkpath('Pdfdata/mmht2014lo135') ! LO grid
        elseif     (pdlabel == 'MMHT_nl') then
            prefix = checkpath('Pdfdata/mmht2014nlo120') ! NLO grid
        elseif     (pdlabel == 'MMHT_nn') then
            prefix = checkpath('Pdfdata/mmht2014nnlo118') ! NNLO grid
        endif
        if (index(prefix,'mstw2008') > 0) then
            call GetAllPDFs(prefix,0,x,xmu,u_val,d_val,u_sea,d_sea, &
            str,sbar,chm,cbar,bot,bbar,gluon,photon)
        else
            call MMHTGetAllPDFs(prefix,0,x,xmu,u_val,d_val,u_sea,d_sea, &
            str,sbar,chm,cbar,bot,bbar,gluon,photon)
        endif
    !-----assign MSTW to standard grid
        fx(0)=gluon/x
        if (ih == 1) then
            fx(1)=(d_val+d_sea)/x
            fx(2)=(u_val+u_sea)/x
            fx(-1)=d_sea/x
            fx(-2)=u_sea/x
            fx(+3)=str/x
            fx(+4)=chm/x
            fx(+5)=bot/x
            fx(-3)=sbar/x
            fx(-4)=cbar/x
            fx(-5)=bbar/x
        elseif(ih == -1) then
            fx(-1)=(d_val+d_sea)/x
            fx(-2)=(u_val+u_sea)/x
            fx(+1)=d_sea/x
            fx(+2)=u_sea/x
            fx(-3)=str/x
            fx(-4)=chm/x
            fx(-5)=bot/x
            fx(+3)=sbar/x
            fx(+4)=cbar/x
            fx(+5)=bbar/x
        endif
             
    elseif ((pdlabel(1:3) == 'mrs') &
         .OR. (pdlabel(2:4) == 'mrs')) then

        if     (pdlabel == 'mrs4nf3') then
            mode=1
            call mrst2004f3(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs4lf3') then
            mode=2
            call mrst2004f3(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs4nf4') then
            mode=1
            call mrst2004f4(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs4lf4') then
            mode=2
            call mrst2004f4(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrstqed') then
            mode=1
            call mrstqed(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon,photon)
        elseif (pdlabel == 'mrs04nl') then
            mode=1
            call mrst2004(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs04nn') then
            mode=2
            call mrst2004(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs02nl') then
            mode=1
            call mrst2002(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs02nn') then
            mode=2
            call mrst2002(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs0119') then
            mode=1
            call mrst2001(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs0117') then
            mode=2
            call mrst2001(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs0121') then
            mode=3
            call mrst2001(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs01_j') then
            mode=4
            call mrst2001(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs01lo') then
        !             write(6,*) 'here'
            mode=1
            call mrstlo(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs99_1') then
            mode=1
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs99_2') then
            mode=2
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs99_3') then
            mode=3
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs99_4') then
            mode=4
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs99_5') then
            mode=5
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs99_6') then
            mode=6
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs99_7') then
            mode=7
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs99_8') then
            mode=8
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs99_9') then
            mode=9
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs9910') then
            mode=10
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs9911') then
            mode=11
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs9912') then
            mode=12
            call mrs99(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98z1') then
            mode=1
            call mrs98(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98z2') then
            mode=2
            call mrs98(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98z3') then
            mode=3
            call mrs98(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98z4') then
            mode=4
            call mrs98(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98z5') then
            mode=5
            call mrs98(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98l1') then
            mode=1
            call mrs98lo(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98l2') then
            mode=2
            call mrs98lo(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98l3') then
            mode=3
            call mrs98lo(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98l4') then
            mode=4
            call mrs98lo(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98l5') then
            mode=5
            call mrs98lo(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs98ht') then
            mode=1
            call mrs98ht(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs96r1') then
            mode=1
            call mrs96(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs96r2') then
            mode=2
            call mrs96(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs96r3') then
            mode=3
            call mrs96(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs96r4') then
            mode=4
            call mrs96(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'hmrs90e') then
            mode=1
            call mrsebh(x,xmu,mode,u_val,d_val,u_sea, &
            s_sea,c_sea,b_sea,gluon)
            d_sea=u_sea
        elseif (pdlabel == 'hmrs90b') then
            mode=2
            call mrsebh(x,xmu,mode,u_val,d_val,u_sea, &
            s_sea,c_sea,b_sea,gluon)
            d_sea=u_sea
        elseif (pdlabel == 'mrs95ap') then
            mode=20
            call mrseb(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        elseif (pdlabel == 'mrs95_g') then
            mode=21
            call mrseb(x,xmu,mode,u_val,d_val,u_sea,d_sea, &
            s_sea,c_sea,b_sea,gluon)
        endif
    !-----assign mrs to standard grid
        fx(-5)=b_sea/x
        fx(-4)=c_sea/x
        fx(-3)=s_sea/x
        fx( 0)=gluon/x
        fx(+3)=fx(-3)
        fx(+4)=fx(-4)
        fx(+5)=fx(-5)
        if (ih == 1) then
            fx(1)=(d_val+d_sea)/x
            fx(2)=(u_val+u_sea)/x
            fx(-1)=d_sea/x
            fx(-2)=u_sea/x
        elseif(ih == -1) then
            fx(-1)=(d_val+d_sea)/x
            fx(-2)=(u_val+u_sea)/x
            fx(+1)=d_sea/x
            fx(+2)=u_sea/x
        endif
        if (kcase==kgg2lep) fx(0)=photon/x

    elseif (pdlabel(1:5) == 'cteq3') then
    !   1      CTEQ3M   Standard MSbar scheme   0.116
    !   3      CTEQ3L   Leading Order           0.116
    !   2      CTEQ3D   Standard DIS scheme     0.116
        if (pdlabel == 'cteq3_m') then
            mode=1
        elseif (pdlabel == 'cteq3_l') then
            mode=2
        elseif (pdlabel == 'cteq3_d') then
            mode=3
        endif
        fx(-5)=Ctq3df(mode,-5,x,xmu,Irt)/x
        fx(-4)=Ctq3df(mode,-4,x,xmu,Irt)/x
        fx(-3)=Ctq3df(mode,-3,x,xmu,Irt)/x
        fx(0)=Ctq3df(mode,0,x,xmu,Irt)/x

        fx(+3)=fx(-3)
        fx(+4)=fx(-4)
        fx(+5)=fx(-5)
        if (ih == 1) then
            fx(-1)=Ctq3df(mode,-2,x,xmu,Irt)/x
            fx(-2)=Ctq3df(mode,-1,x,xmu,Irt)/x
            fx(1)=Ctq3df(mode,+2,x,xmu,Irt)/x+fx(-1)
            fx(2)=Ctq3df(mode,+1,x,xmu,Irt)/x+fx(-2)
        elseif(ih == -1) then
            fx(1)=Ctq3df(mode,-2,x,xmu,Irt)/x
            fx(2)=Ctq3df(mode,-1,x,xmu,Irt)/x
            fx(-1)=Ctq3df(mode,+2,x,xmu,Irt)/x+fx(1)
            fx(-2)=Ctq3df(mode,+1,x,xmu,Irt)/x+fx(2)
        endif

    elseif (pdlabel(1:5) == 'cteq4') then
    !   1      CTEQ4M   Standard MSbar scheme   0.116        1.6      cteq4m.tbl
    !   2      CTEQ4D   Standard DIS scheme     0.116        1.6      cteq4d.tbl
    !   3      CTEQ4L   Leading Order           0.116        1.6      cteq4l.tbl
    !   4      CTEQ4A1  Alpha_s series          0.110        1.6      cteq4a1.tbl
    !   5      CTEQ4A2  Alpha_s series          0.113        1.6      cteq4a2.tbl
    !   6      CTEQ4A3  same as CTEQ4M          0.116        1.6      cteq4m.tbl
    !   7      CTEQ4A4  Alpha_s series          0.119        1.6      cteq4a4.tbl
    !   8      CTEQ4A5  Alpha_s series          0.122        1.6      cteq4a5.tbl
    !   9      CTEQ4HJ  High Jet                0.116        1.6      cteq4hj.tbl
    !   10     CTEQ4LQ  Low Q0                  0.114        0.7      cteq4lq.tbl

        if (pdlabel == 'cteq4_m') then
            mode=1
        elseif (pdlabel == 'cteq4_d') then
            mode=2
        elseif (pdlabel == 'cteq4_l') then
            mode=3
        elseif (pdlabel == 'cteq4a1') then
            mode=4
        elseif (pdlabel == 'cteq4a2') then
            mode=5
        elseif (pdlabel == 'cteq4a3') then
            mode=6
        elseif (pdlabel == 'cteq4a4') then
            mode=7
        elseif (pdlabel == 'cteq4a5') then
            mode=8
        elseif (pdlabel == 'cteq4hj') then
            mode=9
        elseif (pdlabel == 'cteq4lq') then
            mode=10
        endif

        fx(-5)=Ctq4Fn(mode,-5,x,xmu)
        fx(-4)=Ctq4Fn(mode,-4,x,xmu)
        fx(-3)=Ctq4Fn(mode,-3,x,xmu)

        fx(0)=Ctq4Fn(mode,0,x,xmu)

        fx(+3)=Ctq4Fn(mode,+3,x,xmu)
        fx(+4)=Ctq4Fn(mode,+4,x,xmu)
        fx(+5)=Ctq4Fn(mode,+5,x,xmu)
        if (ih == 1) then
            fx(1)=Ctq4Fn(mode,+2,x,xmu)
            fx(2)=Ctq4Fn(mode,+1,x,xmu)
            fx(-1)=Ctq4Fn(mode,-2,x,xmu)
            fx(-2)=Ctq4Fn(mode,-1,x,xmu)
        elseif(ih == -1) then
            fx(1)=Ctq4Fn(mode,-2,x,xmu)
            fx(2)=Ctq4Fn(mode,-1,x,xmu)
            fx(-1)=Ctq4Fn(mode,+2,x,xmu)
            fx(-2)=Ctq4Fn(mode,+1,x,xmu)
        endif

    elseif (pdlabel == 'cteq5l1') then
        fx(-5)=Ctq5L(-5,x,xmu)
        fx(-4)=Ctq5L(-4,x,xmu)
        fx(-3)=Ctq5L(-3,x,xmu)

        fx(0)=Ctq5L(0,x,xmu)

        fx(+3)=Ctq5L(+3,x,xmu)
        fx(+4)=Ctq5L(+4,x,xmu)
        fx(+5)=Ctq5L(+5,x,xmu)

        if (ih == 1) then
            fx(1)=Ctq5L(+2,x,xmu)
            fx(2)=Ctq5L(+1,x,xmu)
            fx(-1)=Ctq5L(-2,x,xmu)
            fx(-2)=Ctq5L(-1,x,xmu)
        elseif(ih == -1) then
            fx(1)=Ctq5L(-2,x,xmu)
            fx(2)=Ctq5L(-1,x,xmu)
            fx(-1)=Ctq5L(+2,x,xmu)
            fx(-2)=Ctq5L(+1,x,xmu)
        endif

    elseif ((pdlabel(1:5) == 'cteq5') .OR. &
        (pdlabel(1:4) == 'ctq5')) then
         
        fx(-5)=Ctq5Pdf(-5,x,xmu)
        fx(-4)=Ctq5Pdf(-4,x,xmu)
        fx(-3)=Ctq5Pdf(-3,x,xmu)
         
        fx(0)=Ctq5Pdf(0,x,xmu)

        fx(+3)=Ctq5Pdf(+3,x,xmu)
        fx(+4)=Ctq5Pdf(+4,x,xmu)
        fx(+5)=Ctq5Pdf(+5,x,xmu)

        if (ih == 1) then
            fx(1)=Ctq5Pdf(+2,x,xmu)
            fx(2)=Ctq5Pdf(+1,x,xmu)
            fx(-1)=Ctq5Pdf(-2,x,xmu)
            fx(-2)=Ctq5Pdf(-1,x,xmu)
        elseif(ih == -1) then
            fx(1)=Ctq5Pdf(-2,x,xmu)
            fx(2)=Ctq5Pdf(-1,x,xmu)
            fx(-1)=Ctq5Pdf(+2,x,xmu)
            fx(-2)=Ctq5Pdf(+1,x,xmu)
        endif

    elseif (pdlabel(1:5) == 'cteq6') then

        fx(-5)=Ctq6Pdf(-5,x,xmu)
        fx(-4)=Ctq6Pdf(-4,x,xmu)
        fx(-3)=Ctq6Pdf(-3,x,xmu)
         
        fx(0)=Ctq6Pdf(0,x,xmu)

        fx(+3)=Ctq6Pdf(+3,x,xmu)
        fx(+4)=Ctq6Pdf(+4,x,xmu)
        fx(+5)=Ctq6Pdf(+5,x,xmu)

        if (ih == 1) then
            fx(1)=Ctq6Pdf(+2,x,xmu)
            fx(2)=Ctq6Pdf(+1,x,xmu)
            fx(-1)=Ctq6Pdf(-2,x,xmu)
            fx(-2)=Ctq6Pdf(-1,x,xmu)
        elseif(ih == -1) then
            fx(1)=Ctq6Pdf(-2,x,xmu)
            fx(2)=Ctq6Pdf(-1,x,xmu)
            fx(-1)=Ctq6Pdf(+2,x,xmu)
            fx(-2)=Ctq6Pdf(+1,x,xmu)
        endif

    elseif (pdlabel(1:4) == 'CT10') then

        fx(-5)=CT10Pdf(-5,x,xmu)
        fx(-4)=CT10Pdf(-4,x,xmu)
        fx(-3)=CT10Pdf(-3,x,xmu)

        fx(0)=CT10Pdf(0,x,xmu)

        fx(+3)=CT10Pdf(+3,x,xmu)
        fx(+4)=CT10Pdf(+4,x,xmu)
        fx(+5)=CT10Pdf(+5,x,xmu)

        if (ih == 1) then
            fx(1)=CT10Pdf(+2,x,xmu)
            fx(2)=CT10Pdf(+1,x,xmu)
            fx(-1)=CT10Pdf(-2,x,xmu)
            fx(-2)=CT10Pdf(-1,x,xmu)
        elseif(ih == -1) then
            fx(1)=CT10Pdf(-2,x,xmu)
            fx(2)=CT10Pdf(-1,x,xmu)
            fx(-1)=CT10Pdf(+2,x,xmu)
            fx(-2)=CT10Pdf(+1,x,xmu)
        endif

    elseif (pdlabel(1:5) == 'CT14.' .OR. pdlabel(1:2) == 'PL') then

        fx(-5)=CT14Pdf(-5,x,xmu)
        fx(-4)=CT14Pdf(-4,x,xmu)
        fx(-3)=CT14Pdf(-3,x,xmu)

        fx(0)=CT14Pdf(0,x,xmu)

        fx(+3)=CT14Pdf(+3,x,xmu)
        fx(+4)=CT14Pdf(+4,x,xmu)
        fx(+5)=CT14Pdf(+5,x,xmu)

        if (ih == 1) then
            fx(1)=CT14Pdf(+2,x,xmu)
            fx(2)=CT14Pdf(+1,x,xmu)
            fx(-1)=CT14Pdf(-2,x,xmu)
            fx(-2)=CT14Pdf(-1,x,xmu)
        elseif(ih == -1) then
            fx(1)=CT14Pdf(-2,x,xmu)
            fx(2)=CT14Pdf(-1,x,xmu)
            fx(-1)=CT14Pdf(+2,x,xmu)
            fx(-2)=CT14Pdf(+1,x,xmu)
        endif

    elseif (pdlabel(1:7) == 'CT14qed') then

        fx(-5)=CT14Pdfqed(-5,x,xmu)
        fx(-4)=CT14Pdfqed(-4,x,xmu)
        fx(-3)=CT14Pdfqed(-3,x,xmu)

        fx(0)=CT14Pdfqed(0,x,xmu)

        fx(+3)=CT14Pdfqed(+3,x,xmu)
        fx(+4)=CT14Pdfqed(+4,x,xmu)
        fx(+5)=CT14Pdfqed(+5,x,xmu)

        if (ih == 1) then
            fx(1)=CT14Pdfqed(+2,x,xmu)
            fx(2)=CT14Pdfqed(+1,x,xmu)
            fx(-1)=CT14Pdfqed(-2,x,xmu)
            fx(-2)=CT14Pdfqed(-1,x,xmu)
        elseif(ih == -1) then
            fx(1)=CT14Pdfqed(-2,x,xmu)
            fx(2)=CT14Pdfqed(-1,x,xmu)
            fx(-1)=CT14Pdfqed(+2,x,xmu)
            fx(-2)=CT14Pdfqed(+1,x,xmu)
        endif
        if (kcase==kgg2lep) fx(0)=CT14Pdfqed(10,x,xmu)

    elseif (pdlabel(1:2) == 'NN') then

        call NNevolvePDF(x,xmu,fxnnpdf)
        fx(-5:5)=fxnnpdf(-5:5)/x
        if (ih == -1) then
            fx(1)=fxnnpdf(-1)/x
            fx(2)=fxnnpdf(-2)/x
            fx(-1)=fxnnpdf(1)/x
            fx(-2)=fxnnpdf(2)/x
        endif
                     
    !--- NEW ATTEMPT
    elseif (pdlabel(1:5) == 'mtung') then
        if     (pdlabel == 'mtungs1') then
            mode=1
        elseif (pdlabel == 'mtunge1') then
            mode=2
        elseif (pdlabel == 'mtungb1') then
            mode=3
        elseif (pdlabel == 'mtungb2') then
            mode=4
        elseif (pdlabel == 'mtungn1') then
            mode=5
        endif
        call mt(x,xmu,mode,u_val,d_val, &
        u_sea,s_sea,c_sea,b_sea,gluon)
        d_sea=u_sea
    !-----assign to standard grid
        fx(-5)=b_sea/x
        fx(-4)=c_sea/x
        fx(-3)=s_sea/x
        fx( 0)=gluon/x
        fx(+3)=fx(-3)
        fx(+4)=fx(-4)
        fx(+5)=fx(-5)
        if (ih == 1) then
            fx(1)=(d_val+d_sea)/x
            fx(2)=(u_val+u_sea)/x
            fx(-1)=d_sea/x
            fx(-2)=u_sea/x
        elseif(ih == -1) then
            fx(-1)=(d_val+d_sea)/x
            fx(-2)=(u_val+u_sea)/x
            fx(+1)=d_sea/x
            fx(+2)=u_sea/x
        endif
         
    else
        write(6,*) 'Unimplemented pdf distribution'
        write(6,*) 'pdlabel= ',pdlabel
        write(6,*) 'Implemented are: ', &
        'mrs4nf3,mrs4lf3,mrs4nf4,mrs4lf4,mrstqed,', &
        'mrs04nl,mrs04nn,mrs02nl,mrs02nn,', &
        'mrs0119,mrs0177,mrs0121,mrs01_j,', &
        'mrs99_1,mrs99_2,mrs99_3,mrs99_4,mrs99_5,mrs99_6,', &
        'mrs99_7,mrs99_8,mrs99_9,mrs9910,mrs9911,mrs9912,', &
        'mrs98z1,', &
        'mrs98z2,', &
        'mrs98z3,', &
        'mrs98z4,', &
        'mrs98z5,', &
        'mrs96r1,', &
        'mrs96r2,', &
        'mrs96r3,', &
        'mrs96r4,', &
        'hmrs90e,', &
        'hmrs90b,', &
        'cteq4_m,', &
        'cteq5_m,', &
        'cteq5_d,', &
        'cteq5_l,', &
        'cteq5l1,', &
        'cteq5hj,', &
        'cteq5hq,', &
        'cteq5f3,', &
        'cteq5f4,', &
        'cteq6l1,', &
        'cteq6_l,', &
        'cteq6_d,', &
        'cteq6_m,', &
        'cteq6l1,', &
        'cteq61m,', &
        'cteq66m,', &
        'mtungs1,', &
        'mtunge1,', &
        'mtungb1,', &
        'mtungb2,', &
        'mtungn1,', &
        'CT10.00,', &
        'CT14.LL,', &
        'CT14.NL,', &
        'CT14qed,', &
        'PLMC.NN,', &
        'NN2.3NL', &
        'NN2.3NN,'

        stop
    endif
          
!--- now perform the corrections for a nucleon beam, if necessary
    if (nucleon) then
    !        write(6,*) 'x,Q',x,xmu
    !        write(6,*) 'before'
    !        write(6,*) 'fx(+5)=',fx(+5)
    !        write(6,*) 'fx(+4)=',fx(+4)
    !        write(6,*) 'fx(+3)=',fx(+3)
    !        write(6,*) 'fx(+2)=',fx(+2)
    !        write(6,*) 'fx(+1)=',fx(+1)
    !        write(6,*) 'fx( 0)=',fx( 0)
    !        write(6,*) 'fx(-1)=',fx(-1)
    !        write(6,*) 'fx(-2)=',fx(-2)
    !        write(6,*) 'fx(-3)=',fx(-3)
    !        write(6,*) 'fx(-4)=',fx(-4)
    !        write(6,*) 'fx(-5)=',fx(-5)
    !--- extract valence and sea components from the parton distributions
        u_sea=fx(-2)
        d_sea=fx(-1)
        u_val=fx(2)-u_sea
        d_val=fx(1)-d_sea
    !--- apply scale factors from eks98r, with a maximum scale of 100 GeV
        xmu_safe=min(xmu,100d0)
        u_val=u_val*eks98r(x,xmu_safe,xA,1)
        d_val=d_val*eks98r(x,xmu_safe,xA,2)
        u_sea=u_sea*eks98r(x,xmu_safe,xA,3)
        d_sea=d_sea*eks98r(x,xmu_safe,xA,4)
        s_sea=fx(3)*eks98r(x,xmu_safe,xA,5)
        c_sea=fx(4)*eks98r(x,xmu_safe,xA,6)
        b_sea=fx(5)*eks98r(x,xmu_safe,xA,7)
        gluon=fx(0)*eks98r(x,xmu_safe,xA,8)
    !--- write new nucleon distributions
        fx(1)=(xZ*(d_val+d_sea)+(xA-xZ)*(u_val+u_sea))/xA
        fx(2)=(xZ*(u_val+u_sea)+(xA-xZ)*(d_val+d_sea))/xA
        fx(-1)=(xZ*d_sea+(xA-xZ)*u_sea)/xA
        fx(-2)=(xZ*u_sea+(xA-xZ)*d_sea)/xA
        fx(-3)=s_sea
        fx(-4)=c_sea
        fx(-5)=b_sea
        fx(+3)=s_sea
        fx(+4)=c_sea
        fx(+5)=b_sea
        fx(0)=gluon
    endif
              
    ffx(:)=real(fx(:))
            
!--- explicitly set some pdfs to zero to ensure that flags work at NNLO
    if (ggonly) then
        ffx(1:5)=0d0    ! remove quark pdfs
        ffx(-5:-1)=0d0  ! remove antiquark pdfs
    endif
    if (noglue) then
        ffx(0)=0d0      ! remove gluon pdfs
    endif

    where (selectpdfs(ibeam_use,:) .eqv. .false.)
        ffx = 0._dp
    end where
            
    return
          
    76 format(' *    (Atomic number, mass) = (Z,A) = (', &
    i4,',',i4,')   *')
          
    end subroutine fdist

#endif


end module
