!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
module ResummationIntegration
    use types
    use qtResummation_params
    implicit none

    public :: resint
    public :: resexpint

    private

    logical, parameter :: boostBorn = .true.
    ! do not set to .true.!
    logical, parameter :: phaseUseQt = .false.

    contains

    function resint(r,wgt)
        use ieee_arithmetic
        use types
        use Scalevar
        use PDFerrors
        use SCET
        use MCFMStorage
        use qtResummation
        use LHAPDF
        use MCFMSetupPlots
        use SafetyCuts, only : passed_smallnew
        implicit none
        include 'src/Inc/constants.f'
        include 'src/Inc/mxpart.f'
        include 'src/Inc/vegas_common.f'
        include 'src/Inc/kprocess.f'
        include 'src/Inc/kpart.f'
        include 'src/Inc/energy.f'
        include 'src/Inc/npart.f'
        include 'src/Inc/dynamicscale.f'
        include 'src/Inc/initialscales.f'
        include 'src/Inc/first.f'
        include 'src/Inc/scalevar.f'
        include 'src/Inc/scale.f'
        include 'src/Inc/facscale.f'
        include 'src/Inc/qcdcouple.f'
        include 'src/Inc/couple.f'
        include 'src/Inc/nlooprun.f'
        include 'src/Inc/x1x2.f'
        include 'src/Inc/xmin.f'
        include 'src/Inc/nf.f'
        include 'src/Inc/nflav.f'
        include 'src/Inc/nwz.f'
        include 'src/Inc/nproc.f'
        include 'src/Inc/ipsgen.f'
        include 'src/Inc/masses.f'
        logical:: bin
        common/bin/bin
        real(dp) :: BrnRat
        common/BrnRat/BrnRat
        integer :: ih1, ih2
        common/density/ih1,ih2

        real(dp) :: resint
        real(dp), intent(in) :: r(mxdim), wgt

        real(dp):: p(mxpart,4),pjet(mxpart,4),xmsq,xmsqMu,xmsqPdf
        real(dp) :: val,pswt,flux

        real(dp) :: qsq
        real(dp) :: p_born(mxpart,4)

        ! functions
        logical :: includedipole
        real(dp) :: ptpure, twomass

        real(dp) :: xjac
        real(dp) :: qt, phi
        real(dp) :: hard(2)
        real(dp) :: res, resexp
        real(dp) :: msq(-nf:nf, -nf:nf)
        real(dp) :: phaseqt
        real(dp) :: safety

        integer :: j,k

        resint = 0._dp

        currentPDF = 0
        currentNd = 0

        p(:,:) = 0._dp
        pjet(:,:) = 0._dp

        qt = r(ndim)*(qtmaxRes-qtminRes) + qtminRes
        phi = 2._dp*pi*r(ndim-1)
        xjac = 2._dp*qt*(qtmaxRes-qtminRes)

!       ! for debugging purposes we can fix qt here
        !qt = 8.0_dp
        !xjac = 2._dp*qt

        if (qt > qtmaxRes) then
            resint = 0._dp
            return
        endif

        if (qt < qtminRes) then
            resint = 0._dp
            return
        endif

        p(:,:) = 0._dp

        if (phaseUseQt) then
            phaseqt = qt
        else
            phaseqt = 0._dp
        endif

        safety = 1d-9

        if (nproc == 1 .or. nproc == 6) then ! W+, W-
            npart = 2
            if (gen_res(r,phaseqt,p,pswt) .eqv. .false. ) then
                resint = 0._dp
            endif
        elseif (any(nproc == [31,32])) then ! Z -> e e
            npart = 2
            ! generate phase space at zero pT
            if (gen_res(r,phaseqt,p,pswt) .eqv. .false. ) then
                resint = 0._dp
            endif
        elseif (kcase == kggfus0) then ! H -> ga ga
            npart = 2
            !if (gen_res(r,phaseqt,p,pswt) .eqv. .false.) then
                !resint = 0._dp
                !return
            !endif
            call gen2m(r,p,pswt,*999)
        elseif (kcase == kHi_Zga) then
            npart=3
            call gen3h(r,p,pswt,*999)
        elseif (kcase == kWHbbar .or. kcase == kWHgaga) then
            npart=4
            call genVH(r,p,pswt,*999)
        elseif (kcase == kZHbbar .or. kcase == kZHgaga) then
            npart=4
            call genVH(r,p,pswt,*999)
        elseif (kcase == kZH__WW) then
            npart=6
            call gen6(r,p,pswt,*999)
        elseif (any(nproc == [300,305])) then ! Z -> e+ e-
            npart = 3
            if (ipsgen == 1) then
                call gen_Vphotons_jets(r,1,0,p,pswt,*999)
            elseif (ipsgen == 2) then
                call genVphoton(r,p,pswt,*999)
            else
              error stop
            endif
            zgaips: block
                real(dp) :: s34, s345, wt34, wt345, wtips(2)
                real(dp) :: dot
              s34 = 2._dp*dot(p,3,4)
              s345 = s34+2._dp*dot(p,3,5)+2._dp*dot(p,4,5)
              wt34 = s34*(s34-zmass**2)**2+(zmass*zwidth)**2
              wt345 = s345*(s345-zmass**2)**2+(zmass*zwidth)**2
              wtips(1) = wt345
              wtips(2) = wt34
              pswt = pswt*wtips(ipsgen)/(wtips(1)+wtips(2))
            end block zgaips
        elseif (nproc == 285 .or. nproc == 2851 .or. nproc == 2852) then ! gamma gamma
            npart = 2
            call gen_photons_jets_res(r,2,0,p,pswt,*999)
            safety = 1d-7
        elseif (kcase == kZZlept) then
            npart = 4
            call gen4(r,p,pswt,*999)
        else
            write (*,*) __FILE__//": undefined nproc, line ", __LINE__
            error stop 
        endif

        if (.not. passed_smallnew(p,npart,safety)) then
 999        continue
            resint = 0._dp
            return
        endif


        if (any(ieee_is_nan(p(1:npart,:)) .eqv. .true.)) then
          !write(6,*) 'Discarding NaN or infinite phase space point'
          resint = 0._dp
          return
        endif


! these are calculated before the boost
        xx(1)=-2._dp*p(1,4)/sqrts
        xx(2)=-2._dp*p(2,4)/sqrts

        if ( (xx(1) > one)  .or. (xx(2) > one) &
         .or.(xx(1) < xmin) .or. (xx(2) < xmin)) then
            resint = 0._dp
            return
        endif

! save before boost for matrix element evaluation
        p_born(1:npart+2,:) = p(1:npart+2,:)
! mostly for plotting purposes
        call recoilBoost(qt, phi, p)

        if (boostBorn) then
            p_born = p
        endif

! check cuts, fills ptildejet
        includeTaucutgrid(0) = .TRUE. 
        if (includedipole(0,p) .eqv. .FALSE. ) then
            resint = 0._dp
            return
        endif

! fill pjet array
        call getptildejet(0,pjet)


        flux = fbGeV2/(two*xx(1)*xx(2)*sqrts**2)

        ! central value
        call computeResint(p, p_born, qt, kresorder, xx, resout=xmsq)

        if (doScalevar .and. bin) then
            do j=1,maxscalevar
                call computeResint(p,p_born,qt,kresorder,xx, resout=xmsqMu, &
                    muMultHard_in=scalevarmult(j), muMultRes_in=facscalevarmult(j))
                scalereweight(j) = xmsqMu / xmsq
            enddo
        endif

        if ((maxPDFsets > 0) .and. bin) then
            pdfreweight(:) = 0._dp
            do j=1,maxPDFsets
                currentPDF = j
                call computeResint(p, p_born, qt, kresorder, xx, resout=xmsqPdf)
                pdfreweight(currentPDF) = flux*xjac*pswt*(xmsq-xmsqPdf)/BrnRat*wgt
            enddo
        endif

        resint = flux*xjac*pswt*xmsq/BrnRat

        !write (*,*) "BrnRat = ", BrnRat
        !write (*,*) "resint = ", resint
        !pause

        val=resint*wgt

        if (ieee_is_nan(val) .OR. ( .NOT. ieee_is_finite(val))) then
            write(6,*) 'Discarded NaN, val=',val
            resint = 0._dp
            return
        endif

        if (bin) then
            includeTaucutgrid(0) = .true. 
            call nplotter_new(pjet,val)
        endif

    end function resint

    subroutine computeResint(p, p_born, qt, order, xx, resout, resexpout, muMultHard_in, muMultRes_in)
        use types
        use qtResummation
        use Scalevar
        implicit none
        include 'src/Inc/nf.f'
        include 'src/Inc/mxpart.f'
        include 'src/Inc/kprocess.f'
        include 'src/Inc/constants.f'
        include 'src/Inc/initialscales.f'
        include 'src/Inc/dynamicscale.f'
        include 'src/Inc/scale.f'
        include 'src/Inc/qcdcouple.f'
        include 'src/Inc/ewcharge.f'
        include 'src/Inc/ewcouple.f'

        real(dp), intent(in) :: p(mxpart,4), p_born(mxpart,4)
        real(dp), intent(in) :: qt
        integer, intent(in) :: order
        real(dp), intent(in) :: xx(2)
        real(dp), intent(out), optional :: resout, resexpout
        real(dp), intent(in), optional :: muMultHard_in, muMultRes_in

        real(dp) :: twomass, threemass, fourmass, puremass
        real(dp) :: gg_2gam_msbar

        integer :: j,k
        real(dp) :: muMultHard, muMultRes, hard(2)
        real(dp) :: xmsq, qsq, msq(-nf:nf,-nf:nf), res, resexp

        real(dp) :: msqall(-nf:nf,-nf:nf,0:2), ggcontrib
        real(dp) :: mtcorr

        integer :: hardorder

        if (present(muMultHard_in)) then
            muMultHard = muMultHard_in
        else
            muMultHard = 1._dp
        endif

        if (present(muMultRes_in)) then
            muMultRes = muMultRes_in
        else
            muMultRes = 1._dp
        endif

        if (dynamicscale) then
            call scaleset(initscale*muMultHard, initfacscale*muMultRes, p)
        else
            call usescales(initscale*muMultHard, initfacscale*muMultRes)
        endif

        msqall = 0._dp

        if (kcase == kW_only) then
            qsq = twomass(1,2,p)**2
            call hardqq(qsq, musq, hard)
            if (boostBorn) then
                call qqb_w(p, msq)
            else
                call qqb_w(p_born, msq)
            endif

            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)
            where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
            where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2

        elseif (kcase == kZ_only) then
            qsq = twomass(1,2,p)**2
            call hardqq(qsq, musq, hard)
            if (boostBorn) then
                call qqb_z(p, msq)
            else
                call qqb_z(p_born, msq)
            endif

            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)
            where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
            where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2
        elseif (kcase == kggfus0) then
            qsq = twomass(1,2,p)**2
            call hardgg(qsq, musq, hard)
            if (boostBorn) then
                call gg_h(p,msq)
            else
                call gg_h(p_born, msq)
            endif
            ! Born with full top-loop
            !call finitemtcorr(mtcorr)
            !msq(0,0) = msq(0,0)*mtcorr
            ! debug
            !as = 0.1126394647838897d0

            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)
            where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
            where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2
        elseif (kcase == kHi_Zga) then
            qsq = threemass(3,4,5,p)**2
            call hardgg(qsq, musq, hard)
            if (boostBorn) then
                call gg_hzgam(p, msq)
            else
                call gg_hzgam(p_born, msq)
            endif

            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)
            where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
            where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2
        elseif (kcase == kWHbbar) then
            qsq = fourmass(3,4,5,6,p)**2
            call hardqq(qsq, musq, hard)
            if (boostBorn) then
                call qqb_wh(p, msq)
            else
                call qqb_wh(p_born, msq)
            endif

            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)
            where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
            where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2
        elseif (kcase == kWHgaga) then
            qsq = fourmass(3,4,5,6,p)**2
            call hardqq(qsq, musq, hard)
            if (boostBorn) then
                call qqb_wh_gaga(p, msq)
            else
                call qqb_wh_gaga(p_born, msq)
            endif

            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)
            where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
            where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2
        elseif (kcase == kZHbbar) then
            qsq = fourmass(3,4,5,6,p)**2
            call hardqq(qsq, musq, hard)
            if (boostBorn) then
                call qqb_zh(p, msq)
            else
                call qqb_zh(p_born, msq)
            endif

            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)
            where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
            where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2
        elseif (kcase == kZH__WW) then
            qsq = puremass(p(3,:)+p(4,:)+p(5,:)+p(6,:)+p(7,:)+p(8,:))**2
            call hardqq(qsq, musq, hard)
            if (boostBorn) then
                call qqb_zh_ww(p, msq)
            else
                call qqb_zh_ww(p_born, msq)
            endif

            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)
            where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
            where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2
        elseif (kcase == kZHgaga) then
            qsq = fourmass(3,4,5,6,p)**2
            call hardqq(qsq, musq, hard)
            if (boostBorn) then
                call qqb_zh_gaga(p, msq)
            else
                call qqb_zh_gaga(p_born, msq)
            endif

            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)
            where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
            where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2
        elseif (kcase == kZgamma) then
            qsq = threemass(3,4,5,p)**2
            if (boostBorn) then
                call zgam_mat(p,msqall)
                call gg_zgam(p,ggcontrib)
            else
                call zgam_mat(p_born,msqall)
                call gg_zgam(p_born,ggcontrib)
            endif

            msqall(0,0, 2) = ggcontrib
            msq(:,:) = msqall(:,:,0)
        elseif (kcase == kgg2gam) then
            qsq = twomass(3,4,p)**2
            if (boostBorn) then
                call gg_gamgam(p,ggcontrib)
                msqall(0,0, 0) = ggcontrib
                msqall(0,0, 1) = gg_2gam_msbar(p)
            else
                call gg_gamgam(p_born,ggcontrib)
                msqall(0,0, 0) = ggcontrib
                msqall(0,0, 1) = gg_2gam_msbar(p_born)
            endif

        elseif (kcase == kgamgam) then
            qsq = twomass(3,4,p)**2
            ggcontrib = 0._dp
            if (boostBorn) then
                call qqb_gamgam(p,msq)
                if (order > 4) then
                    call gg_gamgam(p,ggcontrib)
                    !call gggaga_mt(p,ggcontrib) ! with mt effect
                endif
            else
                call qqb_gamgam(p_born,msq)
                if (order > 4) then
                    call gg_gamgam(p_born,ggcontrib)
                    !call gggaga_mt(p_born,ggcontrib) ! with mt effect
                endif
            endif

            msqall(0,0, 2) = ggcontrib
            
            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)

            gamgam: block
                real(dp) :: qqb, tlrem
                real(dp) :: qsum, fac
                integer :: jj
                
                if (boostBorn) then
                    call gamgamampsq_new(2,p,1,2,3,4,qqb,hard,tlrem)
                else
                    call gamgamampsq_new(2,p_born,1,2,3,4,qqb,hard,tlrem)
                endif

                fac = xn*esq**2*0.5_dp
                qsum = sum(Q(1:5)**2)

                where(msq /= 0._dp) msqall(:,:,1) = hard(1)*msq(:,:)*(as/2._dp/pi)
                where(msq /= 0._dp) msqall(:,:,2) = hard(2)*msq(:,:)*(as/2._dp/pi)**2

                do jj=-5,5
                    if (jj == 0) cycle
                    msqall(jj,-jj,2) = msqall(jj,-jj,2) + fac*aveqq*tlrem* &
                        qsum*Q(abs(jj))**2 *(as/2._dp/pi)**2
                    !write (*,*) jj, msq(jj,-jj) , (fac*aveqq*qqb*Q(abs(jj))**4)
                enddo
            end block gamgam
        elseif (kcase == kZZlept) then
            qsq = fourmass(3,4,5,6,p)**2
            if (boostBorn) then
                call qqb_zz(p,msq)
            else
                call qqb_zz(p_born,msq)
            endif
            where(msq /= 0._dp) msqall(:,:,0) = msq(:,:)

            !msq = 0._dp
            !call qqb_zz_v(p,msq)
            !where(msq /= 0._dp) msqall(:,:,1) = msq(:,:)
        else
            write (*,*) __FILE__//": undefined kcase, line", __LINE__
            error stop
        endif


        if (order < 3) then
           hardorder = 0
        elseif (order < 5) then
           hardorder = 1
        else
           hardorder = 2
        endif

        if (present(resout)) then
            xmsq = 0._dp
            do j=-5,5; do k =-5,5
                if ( all(msqall(j,k,0:hardorder) == 0._dp) ) cycle
                res = 0._dp

                ! hard function set to 0._dp, not required for resummed piece
                call resummation(qsq, qt, xx(1), xx(2), j, k, 0._dp, sqrt(musq), &
                    order, muMult=muMultRes, res=res)

                if (hardorder == 0) then
                   xmsq = xmsq + res * msqall(j,k,0)
                elseif (hardorder == 1) then
                    xmsq = xmsq + res * sum(msqall(j,k,[0,1]))
                elseif (hardorder == 2) then
                    xmsq = xmsq + res * sum(msqall(j,k,[0,1,2]))
                endif
            enddo; enddo

            resout = xmsq
            return
        endif

        if (present(resexpout)) then
            xmsq = 0._dp
            do j=-5,5; do k =-5,5
                ! cycle if no contribution through NLO
                if ( all(msqall(j,k,[0,1]) == 0._dp) ) cycle

                resexp = 0._dp

                ! coefficient in as/4/pi
                hard(1) = msqall(j,k,1) / msqall(j,k,0) / (as/4._dp/pi)

                call resummation(qsq, qt, xx(1), xx(2), j, k, hard(1), sqrt(musq), order, resexp=resexp)

                ! matching correction
                xmsq = xmsq - msqall(j,k,0)*resexp

                !write (*,*) "prefactor", j,k, msqall(j,k,0)*fbgev2
            enddo; enddo

            resexpout = xmsq
            return
        endif


    end subroutine computeResint

    function resexpint(r,wgt)
        use ieee_arithmetic
        use types
        use Scalevar
        use PDFerrors
        use SCET
        use MCFMStorage
        use qtResummation
        use LHAPDF
        use MCFMSetupPlots
        implicit none
        include 'src/Inc/constants.f'
        include 'src/Inc/mxpart.f'
        include 'src/Inc/vegas_common.f'
        include 'src/Inc/kprocess.f'
        include 'src/Inc/kpart.f'
        include 'src/Inc/energy.f'
        include 'src/Inc/npart.f'
        include 'src/Inc/dynamicscale.f'
        include 'src/Inc/initialscales.f'
        include 'src/Inc/first.f'
        include 'src/Inc/scalevar.f'
        include 'src/Inc/scale.f'
        include 'src/Inc/facscale.f'
        include 'src/Inc/qcdcouple.f'
        include 'src/Inc/couple.f'
        include 'src/Inc/nlooprun.f'
        include 'src/Inc/x1x2.f'
        include 'src/Inc/xmin.f'
        include 'src/Inc/nf.f'
        include 'src/Inc/nflav.f'
        include 'src/Inc/nwz.f'
        include 'src/Inc/nproc.f'
        include 'src/Inc/ipsgen.f'
        include 'src/Inc/masses.f'
        logical:: bin
        common/bin/bin
        real(dp) :: BrnRat
        common/BrnRat/BrnRat
        integer :: ih1, ih2
        common/density/ih1,ih2

        real(dp) :: resexpint
        real(dp), intent(in) :: r(mxdim), wgt

        real(dp):: p(mxpart,4),pjet(mxpart,4),xmsq
        real(dp) :: val,pswt,flux

        real(dp) :: qsq
        real(dp) :: p_born(mxpart,4)

        ! functions
        logical :: includedipole
        real(dp) :: ptpure, twomass, threemass

        real(dp) :: xjac
        real(dp) :: qt, phi
        real(dp) :: hard(2)
        real(dp) :: res, resexp, xmsqMu, xmsqPdf
        real(dp) :: msq(-nf:nf, -nf:nf)
        real(dp) :: phaseqt

        real(dp) :: pt34com
        common/pt34com/pt34com
!$omp threadprivate(/pt34com/)

        integer :: j,k

        resexpint = 0._dp

        currentPDF = 0
        currentNd = 0

        p(:,:) = 0._dp
        pjet(:,:) = 0._dp

        qt = r(ndim)*(qtmaxResexp-qtminResexp) + qtminResexp
        phi = 2._dp*pi*r(ndim-1)
        xjac = 2._dp*qt*(qtmaxResexp-qtminResexp)

!       ! for debugging purposes we can fix qt here
        !qt = 1.5_dp
        !xjac = 2._dp*qt

        pt34com = qt

        if (qt > qtmaxResexp) then
            resexpint = 0._dp
            return
        endif

        if (qt < qtminResexp) then
            resexpint = 0._dp
            return
        endif

        p(:,:) = 0._dp

        if (phaseUseQt) then
            phaseqt = qt
        else
            phaseqt = 0._dp
        endif

        if (nproc == 1 .or. nproc == 6) then
            npart = 2
            if (gen_res(r,phaseqt,p,pswt) .eqv. .false. ) then
                resexpint = 0._dp
            endif
        elseif (any(nproc == [31,32])) then ! Z -> e e
            npart = 2
            ! generate phase space at zero pT
            if (gen_res(r,phaseqt,p,pswt) .eqv. .false. ) then
                resexpint = 0._dp
            endif
        elseif (kcase == kggfus0) then ! H -> ga ga
            npart = 2
            if (gen_res(r,phaseqt,p,pswt) .eqv. .false.) then
                resexpint = 0._dp
                return
            endif
        elseif (kcase == kHi_Zga) then
            npart=3
            call gen3h(r,p,pswt,*888)
        elseif (kcase == kWHbbar .or. kcase == kWHgaga) then
            npart=4
            call genVH(r,p,pswt,*888)
        elseif (kcase == kZHbbar .or. kcase == kZHgaga) then
            npart=4
            call genVH(r,p,pswt,*888)
        elseif (kcase == kZH__WW) then
            npart=6
            call gen6(r,p,pswt,*888)
        elseif (any(nproc == [300,305])) then ! Z -> e+ e-
            npart = 3
            if (ipsgen == 1) then
                call gen_Vphotons_jets(r,1,0,p,pswt,*888)
            elseif (ipsgen == 2) then
                call genVphoton(r,p,pswt,*888)
            else
              error stop
            endif
            zgaips: block
                real(dp) :: s34, s345, wt34, wt345, wtips(2)
                real(dp) :: dot
              s34 = 2._dp*dot(p,3,4)
              s345 = s34+2._dp*dot(p,3,5)+2._dp*dot(p,4,5)
              wt34 = s34*(s34-zmass**2)**2+(zmass*zwidth)**2
              wt345 = s345*(s345-zmass**2)**2+(zmass*zwidth)**2
              wtips(1) = wt345
              wtips(2) = wt34
              pswt = pswt*wtips(ipsgen)/(wtips(1)+wtips(2))
            end block zgaips
        elseif (nproc == 285 .or. nproc == 2851 .or. nproc == 2852) then
            npart = 2
            call gen_photons_jets_res(r,2,0,p,pswt,*888)
        elseif (kcase == kZZlept) then
            npart = 4
            call gen4(r,p,pswt,*888)
        else
            write (*,*) __FILE__//": undefined nproc, line ", __LINE__
            error stop 
        endif

        !call smallnew(p,npart,*999)
        if (.false.) then
 888        continue
            resexpint = 0._dp
            return
        endif


        if (any(ieee_is_nan(p(1:npart+2,:)) .eqv. .true.)) then
          !write(6,*) 'Discarding NaN or infinite phase space point'
          resexpint = 0._dp
          return
        endif


! these are calculated before the boost
        xx(1)=-2._dp*p(1,4)/sqrts
        xx(2)=-2._dp*p(2,4)/sqrts

        if ( (xx(1) > one)  .or. (xx(2) > one) &
         .or.(xx(1) < xmin) .or. (xx(2) < xmin)) then
            resexpint = 0._dp
            return
        endif

        !!! DEBUG, pt cut on photon before boost
!       if ( ptpure(p(5,:)) < 20d0 ) then
!           resexpint = 0._dp
!           return
!       endif

! save before boost for matrix element evaluation
        p_born(1:npart+2,:) = p(1:npart+2,:)
! mostly for plotting purposes
        call recoilBoost(qt, phi, p)

        if (boostBorn) then
            p_born = p
        endif

! check cuts, fills ptildejet
        includeTaucutgrid(0) = .TRUE. 
        if (includedipole(0,p) .eqv. .FALSE. ) then
            resexpint = 0._dp
            return
        endif

        call storeptilde(0,p)

!       if ( ptpure(p(3,:)+p(4,:)+p(5,:)) < 69d0 ) then
!           resexpint = 0._dp
!           return
!       endif

! fill pjet array
        call getptildejet(0,pjet)

        if (dynamicscale) then
            call scaleset(initscale, initfacscale, p)
        else
            call usescales(initscale, initfacscale)
        endif

        flux = fbGeV2/(two*xx(1)*xx(2)*sqrts**2)

        ! central value
        xmsq = 0._dp
        call computeResint(p, p_born, qt, kresorder, xx, resexpout=xmsq)

        if (doScalevar .and. bin) then
            do j=1,maxscalevar
                call computeResint(p,p_born,qt,kresorder,xx, resexpout=xmsqMu, &
                    muMultHard_in=scalevarmult(j), muMultRes_in=facscalevarmult(j))
                scalereweight(j) = xmsqMu / xmsq
            enddo
        endif

        if ((maxPDFsets > 0) .and. bin) then
            pdfreweight(:) = 0._dp
            do j=1,maxPDFsets
                currentPDF = j
                call computeResint(p, p_born, qt, kresorder, xx, resexpout=xmsqPdf)
                pdfreweight(currentPDF) = flux*xjac*pswt*(xmsq-xmsqPdf)/BrnRat*wgt
            enddo
        endif

        resexpint = flux*xjac*pswt*xmsq/BrnRat

        !write (*,*) "resexpint = ", resexpint
        !pause

        val=resexpint*wgt

        if (ieee_is_nan(val) .OR. ( .NOT. ieee_is_finite(val))) then
            !write(6,*) 'Discarded NaN, val=',val
            resexpint = 0._dp
            return
        endif
              
        if (bin) then
            includeTaucutgrid(0) = .true. 
            call nplotter_new(pjet,val)
        endif

    end function resexpint

end module
