!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine qqb_wbbm(p,msq)
      implicit none
      include 'src/Inc/types.f'
c--- Note: optimization in the "sumsq" function by RKE, April 2009.
c---  Matrix elements squared 
c     q(-p1)+qb(-p2) --> nu(p3)+e^+(p4)+b(p5)+bb(p6)
c---  averaged(summed) over initial(final) colours and spins
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/ckm.f'
      include 'src/Inc/masses.f'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/heavyflav.f'
      integer:: j,k
      real(dp):: p(mxpart,4),msq(-nf:nf,-nf:nf)
      real(dp):: qqb,qbq,sumsq
      real(dp):: faclo,mQsq
      faclo=V*gsq**2*gwsq**2*aveqq

C--- Set up the correct mass, according to 'flav'
      if     (flav == 6) then
        mQsq=mt**2
      elseif (flav == 5) then
        mQsq=mb**2
      elseif (flav == 4) then
        mQsq=mc**2
      else
        write(6,*) 'Wrong flavour in qqb_wbbm.f: flav=',flav
        stop
      endif

C---Initialize whole array to zero
      msq(:,:)=zero

C---Fill dot-products
      call dotem(6,p,s)

      qqb=faclo*sumsq(1,2,3,4,6,5,mQsq)
      qbq=faclo*sumsq(2,1,3,4,6,5,mQsq)

      do j=-(flav-1),(flav-1)
      do k=-(flav-1),(flav-1)
      if     ((j > 0) .and. (k < 0)) then
               msq(j,k)=Vsq(j,k)*qqb
      elseif ((j < 0) .and. (k > 0)) then
               msq(j,k)=Vsq(j,k)*qbq
      endif
      enddo
      enddo

      return
      end

      function sumsq(p1,p2,p3,p4,p5,p6,mQsq)
      implicit none
      include 'src/Inc/types.f'
      real(dp):: sumsq
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/masses.f'
      integer:: p1,p2,p3,p4,p5,p6
      real(dp):: s56,s134,s234,prop,mQsq
      s134=s(p1,p3)+s(p1,p4)+s(p3,p4)
      s234=s(p2,p3)+s(p2,p4)+s(p3,p4)
      s56=s134+s234+s(p1,p2)-s(p3,p4)
c---overall factor of 4 removed
      prop=s56**2*((s(p3,p4)-wmass**2)**2+(wmass*wwidth)**2)

      sumsq=one/(s134*s234)
     & *(+s(p1,p4)*s(p2,p4)
     & *(s(p2,p6)*s(p3,p5)+s(p2,p5)*s(p3,p6)+two*s(p3,p5)*s(p3,p6))
     & +s(p1,p3)*s(p2,p3)
     & *(s(p1,p6)*s(p4,p5)+s(p1,p5)*s(p4,p6)+two*s(p4,p5)*s(p4,p6))
     & -s(p2,p3)*s(p3,p4)
     & *(s(p1,p6)*s(p4,p5)+s(p1,p5)*s(p4,p6)+two*s(p1,p5)*s(p1,p6))
     & -s(p1,p4)*s(p3,p4)
     & *(s(p2,p6)*s(p3,p5)+s(p2,p5)*s(p3,p6)+two*s(p2,p5)*s(p2,p6)))

      sumsq=sumsq+((s(p1,p2)*s(p3,p4)-s(p1,p3)*s(p2,p4))
     &   *(s(p3,p6)*s(p4,p5)+s(p3,p5)*s(p4,p6)
     &    +s(p2,p6)*s(p4,p5)+s(p2,p5)*s(p4,p6)
     &    +s(p1,p6)*s(p3,p5)+s(p1,p6)*s(p2,p5)
     &    +s(p1,p5)*s(p3,p6)+s(p1,p5)*s(p2,p6))
     &  -s(p1,p4)*s(p2,p3)*(+s(p3,p6)*s(p4,p5)+s(p3,p5)*s(p4,p6)
     &  +s(p1,p6)*s(p2,p5)+s(p1,p5)*s(p2,p6)-two*s(p3,p4)*s(p5,p6))
     & )/(s134*s234)

      sumsq = sumsq + two*s134**(-1)*s234**(-1)*mQsq * (
     &     +(s(p1,p2)*s(p3,p4)-s(p1,p3)*s(p2,p4))
     *     *(s(p1,p2)+s(p3,p4)+s(p1,p3)+s(p2,p4))
     &     +s(p1,p4)*s(p2,p3)*(s(p1,p3)-s(p1,p2)+s(p2,p4)+s(p3,p4)))

      sumsq = sumsq+s(p1,p4)/s134**2 * (
     &     +(s(p1,p3)+s(p3,p4))
     &     *(s(p2,p5)*(s(p1,p6)+s(p4,p6))+s(p2,p6)*(s(p1,p5)+s(p4,p5)))
     &     - s(p1,p4)*(s(p2,p5)*s(p3,p6)+s(p2,p6)*s(p3,p5))
     &     +two*mQsq*(
     &     (s(p1,p2)+s(p2,p4))*(s(p1,p3)+s(p3,p4))-s(p1,p4)*s(p2,p3)))
      sumsq = sumsq + s(p2,p3)/s234**2 * (
     &     +(s(p2,p4)+s(p3,p4))
     &     *(s(p1,p5)*(s(p2,p6)+s(p3,p6))+s(p1,p6)*(s(p2,p5)+s(p3,p5)))
     &     -s(p2,p3)*(s(p1,p5)*s(p4,p6)+s(p1,p6)*s(p4,p5))
     &     +two*mQsq*(
     &     (s(p1,p2)+s(p1,p3))*(s(p2,p4)+s(p3,p4))-s(p1,p4)*s(p2,p3)))

      sumsq=sumsq/prop
      return
      end
