!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine haqgg_mass_tb(i1,i2,i3,i4,msq)
! Matrix element squared for the procoess
!  0 -> H + a(i1) + q(i2) + g(i3) + g(i4)
! with full top mass dependence retained
! Note that the bottom quark is included with the pole mass in the propagator,
! and a running mass mb(mh) in the Yukawa coupling
      implicit none
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/constants.f'
      include 'src/Inc/zprods_com.f'
      include 'src/Inc/masses.f'
      include 'src/ggHgg_mass/mtsq.f'
      include 'src/Inc/ewcouple.f'
      include 'src/Inc/qcdcouple.f'
      integer i1,i2,i3,i4,h1,h2,h3,iq
      real(dp):: msq
      complex(dp) :: amp34(2,2,2),amp43(2,2,2),tmp,
     & amp34t(2,2,2),amp43t(2,2,2),amp34b(2,2,2),amp43b(2,2,2)

      do iq=1,2
      
      if (iq == 1) mtsq=mt**2
      if (iq == 2) mtsq=mb**2
      
      call hgggg_integralfill(i1,i2,i3,i4)

! Ordering of helicities is (quark, gluon, gluon)
! First do (+,-) helicities for (antiquark, quark)
      call haqgg_pmpp(i1,i2,i3,i4,za,zb,amp34(1,2,2))
      call haqgg_pmmp(i1,i2,i3,i4,za,zb,amp34(1,1,2))
      call haqgg_pmpm(i1,i2,i3,i4,za,zb,amp34(1,2,1))
      call haqgg_pmmm(i1,i2,i3,i4,za,zb,amp34(1,1,1))
      
      call haqgg_pmmm(i2,i1,i3,i4,zb,za,tmp)
      amp43(1,2,2)=-tmp
      call haqgg_pmpm(i2,i1,i3,i4,zb,za,tmp)
      amp43(1,1,2)=-tmp
      call haqgg_pmmp(i2,i1,i3,i4,zb,za,tmp)
      amp43(1,2,1)=-tmp
      call haqgg_pmpp(i2,i1,i3,i4,zb,za,tmp)
      amp43(1,1,1)=-tmp

! Now call with za<->zb to get (-,+) helicities for (antiquark, quark)
      call haqgg_pmpp(i1,i2,i3,i4,zb,za,amp34(2,1,1))
      call haqgg_pmmp(i1,i2,i3,i4,zb,za,amp34(2,2,1))
      call haqgg_pmpm(i1,i2,i3,i4,zb,za,amp34(2,1,2))
      call haqgg_pmmm(i1,i2,i3,i4,zb,za,amp34(2,2,2))
      
      call haqgg_pmmm(i2,i1,i3,i4,za,zb,tmp)
      amp43(2,1,1)=-tmp
      call haqgg_pmpm(i2,i1,i3,i4,za,zb,tmp)
      amp43(2,2,1)=-tmp
      call haqgg_pmmp(i2,i1,i3,i4,za,zb,tmp)
      amp43(2,1,2)=-tmp
      call haqgg_pmpp(i2,i1,i3,i4,za,zb,tmp)
      amp43(2,2,2)=-tmp

      amp34(:,:,:)=amp34(:,:,:)*mtsq
      amp43(:,:,:)=amp43(:,:,:)*mtsq
      
      if (iq == 1) then
        amp34t(:,:,:)=amp34(:,:,:)
        amp43t(:,:,:)=amp43(:,:,:)
      endif
      if (iq == 2) then
        amp34b(:,:,:)=amp34(:,:,:)
        amp43b(:,:,:)=amp43(:,:,:)
      endif
      
      enddo

      amp34b(:,:,:)=amp34b(:,:,:)*(3.38_dp/mb)       ! Hack for Yukawa with mb(mh)
      amp43b(:,:,:)=amp43b(:,:,:)*(3.38_dp/mb)       ! Hack for Yukawa with mb(mh)

      amp34(:,:,:)=amp34t(:,:,:)+amp34b(:,:,:)
      amp43(:,:,:)=amp43t(:,:,:)+amp43b(:,:,:)

! Add helicities
      msq=zip
      do h1=1,2
      do h2=1,2
      do h3=1,2
      msq=msq+xn*(abs(amp34(h1,h2,h3))**2+abs(amp43(h1,h2,h3))**2)
     &    -one/xn*abs(amp34(h1,h2,h3)+amp43(h1,h2,h3))**2
      enddo
      enddo
      enddo
      
! Apply overall factor
      msq=msq*V*(as**2)**2/vevsq

      return
      end

