!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function pmpmB34(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^- 3_g^+ 4_g^- +H
!     Implementation of Eq.~(6.17) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      complex(dp):: pmpmB34,pmpmB34symm

      pmpmB34=pmpmB34symm(p1,p2,p3,p4,za,zb)
     &       +pmpmB34symm(p2,p1,p4,p3,zb,za)

      return
      end


      function pmpmB34symm(p1,p2,p3,p4,za,zb)
      implicit none
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/constants.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer p1,p2,p3,p4
      real(dp):: s12,s13,s14,s23,s34,s24,s123,s124,s134,s234,s1234,Delta
      complex(dp):: pmpmB34symm,zab2

      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
      
      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
      s123=s12+s13+s23
      s124=s12+s14+s24
      s134=s13+s14+s34
      s234=s23+s24+s34
      s1234=s12+s13+s14+s23+s24+s34
      
      Delta=(s1234-s12-s34)**2-4._dp*s12*s34

      pmpmB34symm = 
     &  4._dp*zb(p1,p3)*za(p1,p4)**2*s134
     & /(za(p1,p3)*zab2(p1,p3,p4,p1)*zab2(p1,p3,p4,p2)**2)
     & +4._dp*zb(p1,p4)**2*za(p3,p4)*zab2(p1,p3,p4,p1)
     & *zab2(p4,p1,p2,p3)*(two*s123+s124)
     &  /(zb(p1,p2)*zab2(p1,p3,p4,p2)*zab2(p3,p1,p2,p4)**2*Delta)
     &+12._dp*za(p1,p2)*zb(p1,p4)**2*za(p3,p4)*zab2(p4,p1,p2,p3)*s134
     &  /(zab2(p1,p3,p4,p2)*zab2(p3,p1,p2,p4)**2*Delta)
     & +4._dp*zab2(p4,p1,p2,p3)*zab2(p4,p1,p3,p4)
     & /(zab2(p1,p3,p4,p2)**2*zab2(p3,p1,p2,p4))
     & +8._dp*zb(p1,p3)*za(p3,p4)*zab2(p1,p2,p3,p4)*zab2(p4,p1,p2,p3)
     & *(s234-s134)/(zab2(p1,p3,p4,p2)**2*zab2(p3,p1,p2,p4)*Delta)
     &-2._dp*zb(p1,p4)*za(p2,p3)*zab2(p4,p1,p2,p3)
     & /(zab2(p1,p3,p4,p2)*zab2(p3,p1,p2,p4)**2)
     &+4._dp*zb(p1,p4)*za(p2,p3)*zab2(p1,p3,p4,p1)*zab2(p4,p1,p2,p3)
     & *(s123-s124)/(zab2(p1,p3,p4,p2)*zab2(p3,p1,p2,p4)**2*Delta)
     &+3._dp*zab2(p2,p3,p4,p1)*zab2(p4,p1,p2,p3)*(s123-s124)
     & *(s234-s134)*(s134+s234)
     & /(zab2(p1,p3,p4,p2)*zab2(p3,p1,p2,p4)*Delta**2)
     & +zab2(p2,p3,p4,p1)*zab2(p4,p1,p2,p3)
     &  *(s13-5._dp*s14-5._dp*s23+s24-14._dp*s34)
     &  /(zab2(p1,p3,p4,p2)*zab2(p3,p1,p2,p4)*Delta)

      return
      end

