!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function pmpmC1x234(p1,p2,p3,p4,za,zb,msqCoeff)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^- 3_g^+ 4_g^- +H
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/ggHgg_mass/mtsq.f'
      integer p1,p2,p3,p4
      complex(dp):: pmpmC1x234,pmpmC1x234m0,pmpmC1x234m2,msqCoeff
      msqCoeff=pmpmC1x234m2(p1,p2,p3,p4,za,zb)
      pmpmC1x234=pmpmC1x234m0(p1,p2,p3,p4,za,zb)+mtsq*msqCoeff
      return
      end


      function pmpmC1x234m0(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^- 3_g^+ 4_g^- +H
!     Implementation of Eq.~(6.15) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer p1,p2,p3,p4
      real(dp):: s12,s13,s14,s23,s24,s34,s234,s12ps13ps14
      complex(dp):: pmpmC1x234m0,zab2,zab1243,zab1342,zab1234

      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
                          
      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
                          
      s234=s23+s24+s34
      s12ps13ps14=s12+s13+s14

      zab1243=zab2(p1,p2,p4,p3)                    
      zab1342=zab2(p1,p3,p4,p2)
      zab1234=zab2(p1,p2,p3,p4)
      pmpmC1x234m0=s12ps13ps14
     & *(-2._dp*s234*zab1243**2
     & *(zb(p2,p3)**2*zab1234**2+zb(p3,p4)**2*zab1342**2)
     & /(zab1234**3*zab1342**3*zb(p2,p3)*zb(p3,p4))

     & +zb(p1,p3)**2*zb(p3,p4)
     & /(zab1234*zb(p1,p4)*zb(p2,p3)*zb(p2,p4)) 
     & +zb(p1,p3)**2*zb(p3,p2)
     & /(zab1342*zb(p1,p2)*zb(p4,p3)*zb(p4,p2))

     & +za(p2,p4)**2*za(p1,p4)
     & /(zab1342*za(p1,p3)*za(p1,p2)*za(p3,p4))
     & +za(p2,p4)**2*za(p1,p2)
     & /(zab1234*za(p1,p3)*za(p1,p4)*za(p3,p2)))

     & -2._dp*zb(p1,p3)**4/(zb(p1,p2)*zb(p1,p4)*zb(p3,p2)*zb(p3,p4))
      return
      end


